# Mentorenwahl: A fullstack application for assigning mentors to students based on their whishes.
# Copyright (C) 2022  Dominic Grimm
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

require "http/headers"
require "mime"

module Backend
  module Web
    module Controllers
      @[ARTA::Route(path: "/")]
      # GraphQL API controller
      class ApiController < ATH::Controller
        @[ARTA::Get("")]
        def playground : ATH::Response | ATH::Exceptions::HTTPException
          {% if flag?(:development) %}
            ATH::StreamedResponse.new(headers: HTTP::Headers{"content-type" => MIME.from_extension(".html")}) do |io|
              IO.copy(Public.get("index.html"), io)
            end
          {% else %}
            if Backend.config.api.graphql_playground_fully_enabled?
              ATH::StreamedResponse.new(headers: HTTP::Headers{"content-type" => MIME.from_extension(".html")}) do |io|
                IO.copy(Public.get("index.html"), io)
              end
            else
              ATH::Exceptions::ServiceUnavailable.new("GraphQL Playground is not enabled. Please enable it in the backend configuration.")
            end
          {% end %}
        end

        # GraphQL query request data
        struct GraphQLQueryData
          include JSON::Serializable

          # Raw query
          getter query : String

          # Variables
          getter variables = {} of String => JSON::Any

          # Operation name
          getter operation_name : String?
        end

        @[ARTA::Post("")]
        @[ATHA::QueryParam("development")]
        def endpoint(request : ATH::Request, development : Bool = false) : ATH::Response
          {% if flag?(:development) %}
            Log.notice { "Development request icoming" } if development
          {% end %}

          query = GraphQLQueryData.from_json(request.body.not_nil!)

          ATH::StreamedResponse.new(headers: HTTP::Headers{"content-type" => MIME.from_extension(".json")}) do |io|
            Api::Schema::SCHEMA.execute(
              io,
              query.query,
              query.variables,
              query.operation_name,
              Api::Context.new(request.headers, development)
            )
          end
        end
      end
    end
  end
end
