# Mentorenwahl: A fullstack application for assigning mentors to students based on their whishes.
# Copyright (C) 2022  Dominic Grimm
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

require "commander"
require "compiled_license"
require "docker"

require "../backend"

Docker.setup
Backend::Db.init

cli = Commander::Command.new do |cmd|
  cmd.use = "backend"
  cmd.short = "Mentorenwahl backend CLI"

  cmd.run do
    puts cmd.help
  end

  cmd.commands.add do |c|
    c.use = "version"
    c.short = "Prints version"
    c.long = c.short

    c.run do
      puts Backend::VERSION
    end
  end

  cmd.commands.add do |c|
    c.use = "authors"
    c.short = "Prints authors"
    c.long = c.short

    c.run do
      puts Backend::AUTHORS.join("\n")
    end
  end

  cmd.commands.add do |c|
    c.use = "licenses"
    c.short = "Prints licenses of projects used by this programs"
    c.long = c.short

    c.run do
      puts CompiledLicense::LICENSES
    end
  end

  cmd.commands.add do |c|
    c.use = "run"
    c.short = "Run the backend"
    c.long = c.short

    c.run do
      Backend::Runner.new.run
    end
  end

  cmd.commands.add do |c|
    c.use = "register <username> <role>"
    c.short = "Seeds the database with required data"
    c.long = c.short

    c.flags.add do |f|
      f.name = "skif"
      f.long = "--skif"
      f.default = false
      f.description = "User at SKIF"
    end

    c.flags.add do |f|
      f.name = "admin"
      f.long = "--admin"
      f.default = false
      f.description = "Register as admin"
    end

    c.flags.add do |f|
      f.name = "yes"
      f.short = "-y"
      f.long = "--yes"
      f.default = false
      f.description = "Answer yes to all questions"
    end

    c.run do |opts, args|
      username = args[0]
      role = Backend::Db::UserRole.from_string(args[1].underscore)
      unless opts.bool["yes"]
        print "Register '#{username}' as '#{role.to_api}'#{opts.bool["admin"] ? " with admin privileges" : nil}? [y/N] "
        abort unless gets(chomp: true).not_nil!.strip.downcase == "y"
      end

      user = Backend::Db::User.create!(username: username, role: role.to_s, skif: opts.bool["skif"], admin: opts.bool["admin"])
      Backend::Worker::Jobs::CacheLdapUserJob.new(user.id).enqueue

      puts "Done!"
    end
  end
end

Commander.run(cli, ARGV)
